# Copyright (c) 2007, Barry Wark
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import ANN
import numpy as np

from info import __doc__

from numpy.testing import NumpyTest
test = NumpyTest().test

from scikits.ann.version import __version__

class kdtree(ANN._kdtree):
    """Immutable kd-tree wrapper for the Approximate Nearest Neighbor library kd-tree implementation"""
    
    def __init__(self, *args):
        """kdtree(arr, bs=1, split=ANN_KD_SUGGEST)
        
        Construct a new ANN Kd-tree wrapper.
            
        *Parameters *
            arr : {array_like; 2D}
                Array of [observations x dimensions]
            bs : {integer; default=1}
                See ANN documentation
            split : {integer; default=ANN_KD_SUGGEST}
                ANN split rule. See ANN documentation
        """
        
        super(kdtree, self).__init__(*args)
        self.nDim = args[0].shape[1]
    
    
    def knn(self, pts, k=1, eps=0.0):
        """(idx,d2) = knn(pts, k=1, eps=0.0)
        
        Find k-nearest neighbors of one (or more) points.
        
        * Parameters *
            pts : numpy.ndarray
                [nPts x nDimensions]
                nDimensions must be the same as the number of dimensions of the points
                that initialized this kdtree.
            k : int
                Number of nearest neighbors to find
            eps : double
                eps of approximate nearest neighbor search. Use eps=0.0 to find exact
                nearest neighbors.
                Default = 0.0.
                
        * Returns *
            idx : numpy.ndarray
                [nPts x k]
                Each row corresponds to the same row in parameter `pts`. Row gives index into
                kdtree's initialized points of nearest neighbors.
            d2 : numpy.ndarray
                [nPts x k]
                Array of squared distances to the points given in idx.
                
        * Example *
        >>> import scikits.ann as ann
        
        >>> import numpy as np

        >>> k=ann.kdtree(np.array([[0.,0],[1,0],[1.5,2]]))

        >>> k.knn([0,.2],1)
        (array([[0]]), array([[ 0.04]]))

        >>> k.knn([0,.2],2)
        (array([[0, 1]]), array([[ 0.04,  1.04]]))

        >>> k.knn([[0,.2],[.1,2],[3,1],[0,0]],2)
        (array([[0, 1],
               [2, 0],
               [2, 1],
               [1, 2]]), array([[ 0.04,  1.04],
               [ 1.96,  4.01],
               [ 3.25,  5.  ],
               [ 1.  ,  6.25]]))

        >>> k.knn([[0,.2],[.1,2],[3,1],[0,0]],3)
        (array([[ 0,  1,  2],
               [ 2,  0,  1],
               [ 2,  1,  0],
               [ 1,  2, -1]]), array([[  4.00000000e-002,   1.04000000e+000,   5.49000000e+000],
               [  1.96000000e+000,   4.01000000e+000,   4.81000000e+000],
               [  3.25000000e+000,   5.00000000e+000,   1.00000000e+001],
               [  1.00000000e+000,   6.25000000e+000,   1.79769313e+308]]))
        
        """
        
        pts = np.atleast_2d(pts)
        assert(pts.shape[1] == self.nDim)
        
        idx = np.empty((pts.shape[0], k), dtype=np.int_)
        d2 = np.empty_like(idx).astype(np.float_)

        self._knn2(pts, idx, d2, eps)
        
        return (idx, d2)
    



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.31
#
# Don't modify this file, modify the SWIG interface instead.
# This file is compatible with both classic and new-style classes.

import _ANN
import new
new_instancemethod = new.instancemethod
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'PySwigObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static) or hasattr(self,name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError,name

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

import types
try:
    _object = types.ObjectType
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0
del types


class _kdtree(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, _kdtree, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, _kdtree, name)
    def __init__(self, *args): 
        this = _ANN.new__kdtree(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _ANN.delete__kdtree
    __del__ = lambda self : None;
    def _knn2(*args): return _ANN._kdtree__knn2(*args)
    def __repr__(*args): return _ANN._kdtree___repr__(*args)
    def __str__(*args): return _ANN._kdtree___str__(*args)
_kdtree_swigregister = _ANN._kdtree_swigregister
_kdtree_swigregister(_kdtree)






#!/usr/bin/env python
# encoding: utf-8
# Copyright (c) 2007, Barry Wark
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
"""
untitled.py

Created by  on 2007-10-24.
Copyright (c) 2007 Barry Wark. All rights reserved.
"""

import numpy as N
from numpy.testing import NumpyTestCase
from numpy.testing import assert_equal, assert_almost_equal, assert_array_almost_equal

from scikits.ann import kdtree

def checkReturnNumber(k, i):
    """check k returns i from knn"""
    
    pt = N.random.rand(3)
    assert_equal(k.knn(pt, i)[0].size,i)
    assert_equal(k.knn(pt, i)[1].size,i)


def distanceFromOrigin(pts):
    """pts is [observations x dimensions]"""
    
    return N.apply_along_axis(lambda (arr) : N.sqrt(N.power(arr,2).sum()),1,pts)


def checkReturnNN(pts,distances):
    
    pt = N.zeros(pts.shape[1])
    tree = kdtree(pts)
    nn,nn_distances = tree.knn(pt,1)
    nn_distances = N.sqrt(nn_distances)
    
    sorted_distances = N.sort(distances)
    
    for j in xrange(nn.size):
        assert_almost_equal(distanceFromOrigin(N.atleast_2d(pts[nn[j]])), nn_distances[j])
        assert_almost_equal(nn_distances[j],sorted_distances[j])
        assert_almost_equal(nn[j], N.where(distances==sorted_distances[j])[0][0])


class TestANNWrapper(object):
    """Unit tests for ANN wrapper"""
    
    def test_knn_returns_corrent_number_of_neighbors(self):
        
        pts = N.random.rand(10,3)
        
        k = kdtree(pts)
        
        for i in xrange(10):
            yield checkReturnNumber,k,i
    
    
    def test_knn_returns_nearest_neighbor(self):
        
        for nObs in [1,3,10]:
            pts = N.random.rand(nObs,2)
            distances = distanceFromOrigin(pts)
            
            for i in xrange(nObs):
                yield checkReturnNN,pts,distances
            
    
                





if __name__ == '__main__':
    import nose
    nose.run(argv=[__file__,__file__])




#!/usr/bin/env python
# encoding: utf-8
# Copyright (c) 2007, Barry Wark
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
"""
setup.py

Created by Barry Wark on 2007-11-07.
Copyright (c) 2007 Barry Wark. All rights reserved.
"""

import os
import os.path

from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration

def configuration(parent_package='', top_path=None):
    
    config = Configuration('ann', parent_package, top_path)
    
    assert('ANN_LIB' in os.environ)
    assert('ANN_INCLUDE' in os.environ)
    
    ann_library_dir = os.environ['ANN_LIB']
    ann_include_dir = os.environ['ANN_INCLUDE']
    
    #TODO: add flags to statically link libANN
    config.add_extension('_ANN',
                        sources = ['ANN.i'],
                        include_dirs = ['.', ann_include_dir],
                        library_dirs = [ann_library_dir],
                        libraries = ['ANN'],
                        language='c++',
                        swig_opts='-c++',
                        )
    config.add_subpackage('tests')
    
    return config

if __name__ == '__main__':
    setup(configuration=configuration)


#!/usr/bin/env python
# encoding: utf-8

# info.py
# 
# Created by Barry Wark on 2007-11-07.
# Copyright (c) 2007 Barry Wark. All rights reserved.
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__doc_title__ = """The Approximate Nearest Neighbor library"""

__doc__ = """The ANN module provides a numpy-compatible python wrapper around the Approximate Nearest Neighbor library.

The Approximate Nearest Neighbor library is available from http://www.cs.umd.edu/~mount/ANN/.

* Installation *
Download and build the Approximate Nearest Neighbor library. Modify the line in this package's setup.py file so that ANN_ROOT is the path to the root of the Approximate Nearest Neighbor library source tree.

Run ::

    python setup.py build
    python setup.py test
    sudo python setup.py install

from within the source directory.

* Usage *
scikits.ann exposes a single class, kdtree that wraps the Approximate Nearest Neighbor library's kd-tree implementation. kdtree has a single (non-constructor) method, knn that finds the indecies (of the points used to construct the kdtree) of the k-nearest neighbors and the squared distances to those points. A little example will probably be much more enlightening::
    >>> import scikits.ann as ann
    
    >>> import numpy as np

    >>> k=ann.kdtree(np.array([[0.,0],[1,0],[1.5,2]]))

    >>> k.knn([0,.2],1)
    (array([[0]]), array([[ 0.04]]))

    >>> k.knn([0,.2],2)
    (array([[0, 1]]), array([[ 0.04,  1.04]]))

    >>> k.knn([[0,.2],[.1,2],[3,1],[0,0]],2)
    (array([[0, 1],
           [2, 0],
           [2, 1],
           [1, 2]]), array([[ 0.04,  1.04],
           [ 1.96,  4.01],
           [ 3.25,  5.  ],
           [ 1.  ,  6.25]]))

    >>> k.knn([[0,.2],[.1,2],[3,1],[0,0]],3)
    (array([[ 0,  1,  2],
           [ 2,  0,  1],
           [ 2,  1,  0],
           [ 1,  2, -1]]), array([[  4.00000000e-002,   1.04000000e+000,   5.49000000e+000],
           [  1.96000000e+000,   4.01000000e+000,   4.81000000e+000],
           [  3.25000000e+000,   5.00000000e+000,   1.00000000e+001],
           [  1.00000000e+000,   6.25000000e+000,   1.79769313e+308]]))
    
"""




#! /usr/bin/env python

# Copyright (c) 2007, Barry Wark
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import setuptools

long_description = """
The ANN module provides a numpy-compatible python wrapper around the 
Approximate Nearest Neighbor library (http://www.cs.umd.edu/~mount/ANN/).

* Installation *
Download and build the Approximate Nearest Neighbor library. Modify the ANN section of 
site.cfg so that ANN_ROOT is the path to the root of the Approximate Nearest Neighbor 
library include/lib tree.
If /usr/local/include contains the ANN/ include directory and /usr/local/lib contains 
libANN.a, then
ANN_ROOT = /usr/local

Run ::

    python setup.py build_ext --inplace build test
    sudo python setup.py install

from within the source directory.

* Usage *
scikits.ann exposes a single class, kdtree that wraps the Approximate Nearest Neighbor 
library's kd-tree implementation. kdtree has a single (non-constructor) method, knn that 
finds the indecies (of the points used to construct the kdtree) of the k-nearest neighbors
 and the squared distances to those points. A little example will probably be much 
 more enlightening::
    >>> import scikits.ann as ann
        
    >>> import numpy as np

    >>> k=ann.kdtree(np.array([[0.,0],[1,0],[1.5,2]]))

    >>> k.knn([0,.2],1)
    (array([[0]]), array([[ 0.04]]))

    >>> k.knn([0,.2],2)
    (array([[0, 1]]), array([[ 0.04,  1.04]]))

    >>> k.knn([[0,.2],[.1,2],[3,1],[0,0]],2)
    (array([[0, 1],
           [2, 0],
           [2, 1],
           [1, 2]]), array([[ 0.04,  1.04],
           [ 1.96,  4.01],
           [ 3.25,  5.  ],
           [ 1.  ,  6.25]]))

    >>> k.knn([[0,.2],[.1,2],[3,1],[0,0]],3)
    (array([[ 0,  1,  2],
           [ 2,  0,  1],
           [ 2,  1,  0],
           [ 1,  2, -1]]), array([[  4.00000000e-002,   1.04000000e+000,   5.49000000e+000],
           [  1.96000000e+000,   4.01000000e+000,   4.81000000e+000],
           [  3.25000000e+000,   5.00000000e+000,   1.00000000e+001],
           [  1.00000000e+000,   6.25000000e+000,   1.79769313e+308]]))
    
"""

classifiers = ['Development Status :: 4 - Beta',
                'Intended Audience :: Developers',
                'Intended Audience :: Science/Research',
                'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
                'Programming Language :: Python',
                'Topic :: Scientific/Engineering',
                'Topic :: Software Development :: Libraries :: Python Modules']

DISTNAME = 'scikits.ann'
VERSION = '0.2'

import os
import os.path
import ConfigParser

config = ConfigParser.ConfigParser()
config.read('site.cfg')

if('ANN_LIB' not in os.environ):
    os.environ['ANN_LIB'] = config.get('ANN', 'ANN_LIB')

if('ANN_INCLUDE' not in os.environ):
    os.environ['ANN_INCLUDE'] = config.get('ANN', 'ANN_INCLUDE')


def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    
    config = Configuration(None, parent_package, top_path,
                            namespace_packages=['scikits'])
    
    config.set_options(
        ignore_setup_xxx_py=True,
        assume_default_configuration=True,
        delegate_options_to_subpackages=True,
        quiet=True,
    )
    
    config.add_subpackage('scikits')
    config.add_subpackage(DISTNAME)
    
    config.add_data_files('scikits/__init__.py')

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    
    setup(
        configuration = configuration,
        name = DISTNAME,
        version = VERSION,
        description = "Approximate Nearest Neighbor library wrapper for Numpy",
        long_description = long_description,
        author      = "Barry Wark",
        author_email = "barrywark@gmail.com",
        install_requires = ['configobj'],
        requires = ['numpy'],
        classifiers = classifiers,
        license = 'GNU Library or Lesser General Public License (LGPL)',
        url = 'http://scipy.org/scipy/scikits/wiki/AnnWrapper',
        test_suite = 'nose.collector',
    )
